#include <avr/io.h>
#include <stdint.h>
#include <stdio.h>
#include "SED1520.h"
#include "graphic.h"
#include <avr/pgmspace.h>
#include "startlogo.h"
#include "ADC.h"
#define F_CPU 8000000UL
#include <util/delay.h>
#include "Config.h"
#include <stdbool.h>

uint16_t drehzahl;
uint16_t temperatur;
uint16_t spannung; /* in 1/10 V */
bool reserve;

void wait_ms(int ms)
{
	int i; 
	for(i = 0; i < ms; i++) _delay_ms(1); 
}

void init(void)
{
	/* Display initialisieren und Startlogo anzeigen */
	GLCD_Init();
	GLCD_Bitmap(startlogobmp, 0, 0, 122, 32);

	ADC_Init();

	/* Pins fr Taster als Eingang defnieren*/
	/*SWITCHDDR &= ~( 1 << RESPIN );*/
	SWITCHDDR = 0x00;

	/* Das Bootlogo ist so schn, das soll man auch sehen */
	wait_ms(BOOTLOGOTIME);

}

void showFrame(void)
{

	GLCD_ClearScreen();
	
	GLCD_GoTo(0,0); 
	GLCD_WriteString("     V");	

	GLCD_GoTo(0,1); 
	GLCD_WriteString("     U/min");	


	GLCD_GoTo(0,2); 
	GLCD_WriteString("     #C");	

}

void showdata(void)
{

	char drehzahl_s[5];
	char temperatur_s[4];
	char spannung_s[4];
	
	/* Spannung */
	sprintf(spannung_s, "%3d.%1d", (spannung / 10), (spannung % 10));

	GLCD_GoTo(0,0); 
	GLCD_WriteString(spannung_s);
	
	/* Drehzahl */
	sprintf(drehzahl_s, "%5u",drehzahl);
	
	GLCD_GoTo(0,1); 
	GLCD_WriteString(drehzahl_s);

	/* Temperatur */
	sprintf(temperatur_s, "%3d.%1d", (temperatur / 10), (temperatur % 10));

	GLCD_GoTo(0,2); 
	GLCD_WriteString(temperatur_s);
	
	/* Reserve */
	GLCD_GoTo(0,3); 
	if (reserve)
	{		
		GLCD_WriteString("        ");
	}
	else
	{
		GLCD_WriteString("Reserve");		
	}

	
	
}

void readReserve(void)
{
	
	reserve = (SWITCHPIN & (1 << RESPIN));

}

void readTemperatur(void)
{

	uint16_t adcvalue;
	
	adcvalue = ADC_Read(TEMPPIN);
	adcvalue = ADC_Read(TEMPPIN);


	/* TODO: Anpassen an den verwendeten NTC 
	   Aufpassen, die Temperatur soll in 1/10 C gespeichert werden
	*/
	
	temperatur = adcvalue;

}

void readSpannung(void)
{

	uint16_t adcvalue;
	
	adcvalue = ADC_Read(SPANNUNGPIN);
	adcvalue = ADC_Read(SPANNUNGPIN);


	/*
	  Berechung der eingelesenen Spannung aus dem ADC-Wert:
	  Uout = ADCValue * (Referenzspannung / 1024)
	  Uout = ADCValue * (ADCREF / 1024)

	  Jetzt noch den Spannungsteiler rausrechnen

	*/

	spannung = (adcvalue * 10 * (ADCREF / 1024));
	

}

void readDrehzahl(void)
{
	uint16_t adcvalue;
	float drehzahl_f;
	
	adcvalue = ADC_Read(DREHPIN);
	adcvalue = ADC_Read(DREHPIN);


	/*

	  Berechnung der Spannung vom Frequenz-Spannungswandler:
	  Uout = fin * VCC * C2 * R4
      Uout = fin * 5V * 0,000.000.047F * 47.000 Ohm
      Uout = fin * 0,011045

	  fin (hertz) = Uout / 0,011045
	  fin (u/min) = Uout / 0,011045 * 60

	  Berechung der eingelesenen Spannung aus dem ADC-Wert:
	  Uout = ADCValue * (Referenzspannung / 1024)
	  Uout = ADCValue * (ADCREF / 1024)

	  Somit folgt:
	  fin = ((ADCValue * (ADCREF / 1024)) / 0,011045) * 60
		
    */

	drehzahl_f = ((adcvalue * (ADCREF / 1024)) / 0.011045) * 60;
	drehzahl = drehzahl_f;

}

int main(void)
{  

	init();
	showFrame();

	while (1)
	{
		/* Hauptarbeitsschleife */
		readSpannung();
		readDrehzahl();
		readTemperatur();
		readReserve();
		showdata();
	
	}
	return 0; 
}






